-- original code by upsidedown (driveControl mod)

manMotorStart = {};

function manMotorStart:prerequisitesPresent(specializations)
    return true;
end;

function manMotorStart:load(xmlFile)
	self.manMotorStart = {};
	self.manMotorStart.noEnterCnt = 0;	
	self.manMotorStart.isActive = true;
	self.manMotorStart.isMotorStarted = false;
	self.manMotorStart.startCnt = 0;
	self.manMotorStart.wasHired = false;
	self.manMotorStart.motorMinRpm = self.motor.minRpm;	
	
	if not manMotorStart.overlayBattery then
        local iconWidth = math.floor(0.0095 * g_screenWidth) / g_screenWidth;
        local iconHeight = iconWidth * g_screenAspectRatio;
        local xAdjust = iconWidth * 0.25
        local yAdjust = iconHeight * 0.25
    
        local x,y = g_currentMission.speedHud.x,g_currentMission.speedHud.y;
        local w,h = g_currentMission.speedHud.width,g_currentMission.speedHud.height;
        local xIndent,yIndent = 0,0
    
        manMotorStart.overlayBattery = Overlay:new("overlayBattery", Utils.getFilename("scripts/battery.dds", self.baseDirectory), x+w-xIndent-iconWidth-xAdjust,y+h-yIndent-iconHeight+yAdjust, iconWidth,iconHeight);
        manMotorStart.overlayPreHeat = Overlay:new("overlayPreHeat", Utils.getFilename("scripts/preHeat.dds", self.baseDirectory), x+w-xIndent-iconWidth-xAdjust,y+h-yIndent-iconHeight+yAdjust, iconWidth,iconHeight);
	end;
	if self.isClient then
		self.sampleZSdrehen = Utils.loadSample(xmlFile, {}, "vehicle.zuendSchluessel", "sounds/zuendschluessel_dreh.wav", self.baseDirectory, self.components[1].node);
	end;
end;

function manMotorStart:postLoad()
	if self.dCdeactivateModule ~= nil then
		self:dCdeactivateModule("manMotorStart");
	end;
end;

function manMotorStart:motorStop(self,doStopSounds)	
	self.isMotorStarted = false;
	if Utils.getNoNil(doStopSounds,false) then 
		self:stopMotor(true);
	else
		Motorized.stopSounds(self);
		if self.exhaustParticleSystems ~= nil then
			Utils.setEmittingState(self.exhaustParticleSystems, false)
		end;
		
		if self.exhaustEffects~= nil then
			for _,effect in pairs(self.exhaustEffects) do
				setVisibility(effect.node,false);
			end;
		end;
	end;
	
	self.motor.minRpm = 0;
	--print("motor turned off on enter")
end;
function manMotorStart:motorStart(self)
	-- self:startMotor(g_server == nil);
	self:startMotor(true);
	if self.exhaustParticleSystems ~= nil then
		Utils.setEmittingState(self.exhaustParticleSystems, true)
	end;
	
	if self.exhaustEffects~= nil then
		for _,effect in pairs(self.exhaustEffects) do
			setVisibility(effect.node,true);
		end;
	end;
	self.motor.minRpm = self.manMotorStart.motorMinRpm;
end;

function manMotorStart:onLeave()
		local vehicles = manMotorStart.getAllConnectedVehicles(self);
		for k,vehicle in pairs(vehicles) do
			vehicle.forceIsActive = self.manMotorStart.isMotorStarted;
		end;
		if self.manMotorStart.lastLightMask ~= nil then
			if self.LA==nil then --lights addon is installed. Let LA do whatever LA does with the lights...
				self:setLightsTypesMask(self.manMotorStart.lastLightMask, true);
			end;
		end;
		
		if self.LA~=nil then --help Light Addon to handle beacons and turn signals
			self.beaconLightsActive = self.LA.beaconActive;
			self:setState("beacon", self.beaconLightsActive);
			self:setBeaconLightsVisibility(not self.LA.beaconActive, true);
			self:setBeaconLightsVisibility(self.LA.beaconActive, true);
			
			if self.turnSignals ~= nil then
				self:setTurnSignalState(not self.LA.turnState, true);
				self:setTurnSignalState(self.LA.turnState, true);
			end;
		end;
	self.axisForward = 0;
	if self.isServer then
		Drivable.updateVehiclePhysics(self, 0, false, 0, false, 15)
	end;
end;

function manMotorStart:onEnter()
	if not self.manMotorStart.isMotorStarted then
		manMotorStart:motorStop(self,false)
	end;
end;

function manMotorStart:mouseEvent(posX, posY, isDown, isUp, button)
end;

function manMotorStart:keyEvent(unicode, sym, modifier, isDown)
end;

function manMotorStart:updateTick(dt)
		self.stopMotorOnLeave = false;
		self.deactivateLightsOnLeave = false;
		self.deactivateOnLeave = false;
	
		self.forceIsActive = self.manMotorStart.isMotorStarted;
		for _,implement in pairs(self.attachedImplements) do
			implement.object.forceIsActive = self.manMotorStart.isMotorStarted;
		end;
		if not self.manMotorStart.isMotorStarted then
			if self:getIsActive() then				
				local vehicles = manMotorStart.getAllConnectedVehicles(self);
				for k,vehicle in pairs(vehicles) do
					if vehicle.isTurnedOn then
						if vehicle.setIsTurnedOn~= nil then
							vehicle:setIsTurnedOn(false, true);
						end;
					end;
				end;
			end;
		end;
		
	if self.isHired then	
		self.manMotorStart.isMotorStarted = true;		
		self.manMotorStart.wasHired = true;		
	else
		if self.manMotorStart.wasHired then				
			if not self.isEntered then
				self.manMotorStart.isMotorStarted = false;		
			end;
		end;
		self.manMotorStart.wasHired = false;
	end;
	
	self.manMotorStart.lastLightMask = self.lightsTypesMask; 
	if self.fuelFillLevel < 0.1 then
		self.manMotorStart.isMotorStarted = false;
	end;
	
	if self.manMotorStart.isMotorStarted ~= self.isMotorStarted then
		if self.manMotorStart.isMotorStarted then --turn on
			manMotorStart:motorStart(self)
		else --turn off
			manMotorStart:motorStop(self,true)
		end;
	end;
end;

function manMotorStart:update(dt)
	if self.isEntered and self:getIsActiveForInput(false) then
		if InputBinding.hasEvent(InputBinding.manMotorStartMotor) then
			if self.manMotorStart.isMotorStarted then
				self.manMotorStart.isMotorStarted = false;
				self.manMotorStart.startCnt = 0;
				manMotorStartInputEvent.sendEvent(self);
			else
				if self.manMotorStart.startCnt == 0 and self.isClient and self:getIsActiveForSound() then
					Utils.playSample(self.sampleZSdrehen,1,1,nil);
				end;
				self.manMotorStart.startCnt = self.manMotorStart.startCnt + 1;
				if self.manMotorStart.startCnt >= 2 then
					self.manMotorStart.isMotorStarted = true;
					manMotorStartInputEvent.sendEvent(self)
					self.manMotorStart.startCnt = 0;
				end;
			end;
		end;
	end;
end;

function manMotorStart:draw()
	if not self.manMotorStart.isMotorStarted then
		if self.manMotorStart.startCnt == 1 then
			manMotorStart.overlayPreHeat:render();
		elseif self.manMotorStart.startCnt == 0 then
			manMotorStart.overlayBattery:render();
		end;
	end;		
end;

function manMotorStart:writeStream(streamId, connection)
	streamWriteBool(streamId, self.manMotorStart.isMotorStarted);
end;

function manMotorStart:readStream(streamId, connection)
	local isMotorStarted = streamReadBool(streamId);
	self.manMotorStart.isMotorStarted = isMotorStarted;
end;

function manMotorStart:delete()	
end;

function manMotorStart.getAllConnectedVehicles(self)	
	local res = {};
	res[1] = self;	
	if self.attachedImplements ~= nil then
		if table.getn(self.attachedImplements) > 0 then			
			for k,implement in pairs(self.attachedImplements) do	
				-- print(implement.object.typeName)
				local returnValue = manMotorStart.getAllConnectedVehicles(implement.object);
				for kk,value in pairs(returnValue) do				
					table.insert(res,value)
				end;
			end;
		end;
	end;	
	return res;	
end

manMotorStartInputEvent = {};
manMotorStartInputEvent_mt = Class(manMotorStartInputEvent, Event);

InitEventClass(manMotorStartInputEvent, "manMotorStartInputEvent");

function manMotorStartInputEvent:emptyNew()
    local self = Event:new(manMotorStartInputEvent_mt);
    self.className="manMotorStartInputEvent";
    return self;
end;

function manMotorStartInputEvent:new(vehicle)
    local self = manMotorStartInputEvent:emptyNew()
    self.vehicle = vehicle;
	self.isMotorStarted = vehicle.manMotorStart.isMotorStarted;
    return self;
end;

function manMotorStartInputEvent:writeStream(streamId, connection)
    streamWriteInt32(streamId, networkGetObjectId(self.vehicle));
	streamWriteBool(streamId, self.isMotorStarted);
	-- print("event writeStream")
end;

function manMotorStartInputEvent:readStream(streamId, connection)
    local id = streamReadInt32(streamId);
    local vehicle = networkGetObject(id);
	local isMotorStarted = streamReadBool(streamId);
	vehicle.manMotorStart.isMotorStarted = isMotorStarted;
	if g_server ~= nil then	
		g_server:broadcastEvent(manMotorStartInputEvent:new(vehicle), nil, nil, vehicle);
		-- print("broadcasting")
	end;
end;

function manMotorStartInputEvent.sendEvent(vehicle)
	if g_server ~= nil then	
		g_server:broadcastEvent(manMotorStartInputEvent:new(vehicle), nil, nil, vehicle);
		-- print("broadcasting")
	else
		g_client:getServerConnection():sendEvent(manMotorStartInputEvent:new(vehicle));
		-- print("sending event to server...")
	end;
end;